﻿/*
*
* Základní datové typy
*
*/

-- Základní datové typy
/* 
  Poznámka :
  <hodnota>::<datovy typ> - Znamená, že hodnota se retypuje na daný datový typ
  cast(<hodnota> as <datovy typ>) - ekvivalent pretypovani

*/

/*
*  Jednoduché datové typy
*  ======================
*/


-- číselné
select 123, 123::numeric;

/*
123 - defualtně interpretováno jako integer
123::numeric - interpretováno jako numeric
*/

-- znakové
select 'ahoj', 'ahoj'::text, cast('ahoj' as varchar),  

-- datumové
select 2016-01-07,'2016-01-07'::date; 

/*

Pozor:
Bez apostrofů se jedna o číselný výraz

*/

-- logické
select true, 'true'::boolean, false;

/*
 Pozor:
 t - není true pouze zkratka pro true obdobně f
*/ 

-- XML
select '<book><title>pozdrav</title></book>'::xml;


-- JSON
select '{"book":{"title":"pozdrav"}}'::json;


/*
*  Složené datové typy
*  ===================
*/

-- =========================
-- ENUM - výčtový datový typ
-- =========================

CREATE TYPE odpoved AS ENUM('ano','možná','ne');

-- Příklad vytvoření tabulky s enum
create table anketa
( id serial primary key,
  dotaz text unique not null,
  hodnota odpoved
 );

-- Vložení dat
insert into anketa
   (dotaz,hodnota)
values 
   ('Miluješ mě ?','možná');

-- Výběr dat
select * from anketa;  

/*
-- Vysledek

 id |    dotaz     | hodnota
----+--------------+---------
  1 | Miluješ mě ? | možná
(1 řádka)
*/

-- Nakonec zrušení db objektů
DROP TABLE anketa;
DROP TYPE odpoved;

-- ====
-- Pole
-- ====

create table pocitani
(id serial primary key,
jazyk text unique,
 hodnota text[]);

-- Vložení dat
insert into pocitani
(jazyk,hodnota)
values
('česky','{"jedna","dva","tři"}');


-- Výběr dat
select jazyk,hodnota[2],hodnota from pocitani;

-- Nakonec zrušní tabulky
drop table pocitani;

-- ======
-- Rozsah
-- ======

-- Příklad vytvoření tabulky
-- událost a kdy trvala
create table event
(id serial primary key,
 event_name text not null unique,
 event_interval daterange not null
 );

-- Vloží se nové události a kdy se stali
insert into event
(event_name,event_interval)
values
('event1','[2001-04-10,2005-10-01]'),
('event2','[2000-08-10,2003-01-20]'),
('event3','[2002-05-01,2009-11-8]');

-- Výběr dat, které jsou zahrnutý v daném intervalu
select *,
event_interval * '[2000-03-1, 2001-05-01]'::daterange as prunik_intervalu,
lower(event_interval) as datum_od,
upper(event_interval) as datum_do
from event
where event_interval && '[2000-03-1, 2001-05-01]'::daterange;

drop table event;

-- ========== 
-- Kompositní
-- ========== 

-- Vytvoření kopositního datového typu
create type clovek as
  (jmeno    text,
   prijmeni text,
   datum_narozeni date);

-- Vytvoření tabulky s kompositním datovým typem
create table mesic_dochazka 
(id serial primary key,
 zamestnanec clovek not null unique,
 odprac_hod numeric not null);

-- Vložení kompositních dat
insert into mesic_dochazka
(zamestnanec,odprac_hod)
values
(('Jmeno1','Prijmeni1','1989-12-1'),200),
(('Jmeno2','Prijmeni2','1994-5-6'),189);

insert into mesic_dochazka
(zamestnanec.jmeno,zamestnanec.prijmeni,odprac_hod)
values
('Jméno 3','Jméno 4',200);

-- Výběr dat
select (zamestnanec).jmeno,(zamestnanec).prijmeni,(zamestnanec).datum_narozeni,odprac_hod
 from mesic_dochazka;

-- Zrušení tabulky a kompositního typu
drop table mesic_dochazka;
drop type clovek;
 

