﻿/*
*
*  XML datové typy
*
*/

-- ===================
-- Kontrola datového typu XML
-- ===================

select 
'<hmotnost>
   <jednotka>
	<nazev>kilogram</nazev><zkr>kg</zkr><koeficient>1</koeficient>
   </jednotka>
   <jednotka>
	<nazev>gram</nazev><zkr>gr</zkr><koeficient>0.001</koeficient>
   </jednotka>  
   <jednotka>
	<nazev>mikrogram</nazev><zkr>μg</zkr><koeficient>0.000000001</koeficient>
   </jednotka>
   <jednotka>
	<nazev>tuna</nazev><zkr>t</zkr><koeficient>1000</koeficient>
   </jednotka>
 </hmotnost>'::XML;

-- ============================================
-- Tabulka obsahuje v jednom sloupci různá XML dle různých XSD
-- ============================================

-- Vytvoření tabulky jednotka

drop table if exists jednotka;
create table jednotka
(id serial primary key,
 typ varchar(80) not null unique,
 hodnota xml not null);


-- Vložení dat
-- První záznam jsou jednotky pro hmotnost
insert into jednotka
(typ,hodnota)
values
(
'hmotnost',
'<hmotnost>
   <jednotka>
	<nazev>kilogram</nazev><zkr>kg</zkr><koeficient>1</koeficient>
   </jednotka>
   <jednotka>
	<nazev>gram</nazev><zkr>gr</zkr><koeficient>0.001</koeficient>
   </jednotka>  
   <jednotka>
	<nazev>mikrogram</nazev><zkr>μg</zkr><koeficient>0.000000001</koeficient>
   </jednotka>
   <jednotka>
	<nazev>tuna</nazev><zkr>t</zkr><koeficient>1000</koeficient>
   </jednotka>
 </hmotnost>' :: XML
 ); 

-- druhý záznam jsou jednotky pro čas
insert into jednotka
(typ,hodnota)
 values
 (
 'cas',
 '<cas>
   <jednotka>
	<nazev>sekunda</nazev><zkr>s</zkr><koeficient>1</koeficient>
   </jednotka>     
   <jednotka>
	<nazev>minuta</nazev><zkr>min</zkr><koeficient>60</koeficient>
   </jednotka>
   <jednotka>
	<nazev>den (střední sluneční)</nazev><zkr>d</zkr><koeficient>86400</koeficient>
   </jednotka>
   <jednotka>
	<nazev>sol (střední Mars)</nazev><zkr>sol</zkr><koeficient>88775</koeficient>
   </jednotka> 
  </cas>':: XML
  );

-- delete from jednotka;


-- Výběr času
-- Vyber ciselniku casu

select 
array_to_string(xpath('/jednotka/nazev/text()'::text, radek),',') as nazev, 
array_to_string(xpath('/jednotka/zkr/text()'::text, radek),',') as zkr,
array_to_string(xpath('/jednotka/koeficient/text()'::text, radek),',')::numeric as koeficient  
from 
(select unnest(xpath('/cas/jednotka'::text,hodnota)) as radek
 from jednotka where typ = 'cas') a;

 /*
         nazev          | zkr | koeficient
------------------------+-----+------------
 sekunda                | s   |          1
 minuta                 | min |         60
 den (střední sluneční) | d   |      86400
 sol (střední Mars)     | sol |      88775
(4 řádky)
*/

-- Výběr hmotnosti

select 
array_to_string(xpath('/jednotka/nazev/text()'::text, radek),',') as nazev, 
array_to_string(xpath('/jednotka/zkr/text()'::text, radek),',') as zkr,
array_to_string(xpath('/jednotka/koeficient/text()'::text, radek),',')::numeric as koeficient  
from 
(select unnest(xpath('/hmotnost/jednotka'::text,hodnota)) as radek
 from jednotka where typ = 'hmotnost') a;

 /*
         nazev          | zkr | koeficient
------------------------+-----+------------
"kilogram";"kg";1
"gram";"gr";0.001
"mikrogram";"μg";0.000000001
"tuna";"t";1000
(4 řádky)
*/


-- ==================================================
-- Tabulka obsahuje v jednom sloupci stejne XML dle různých XSD
-- ==================================================

/*
Příklad obsahuje příklad meteo data předpovědí 
*/

-- drop table weather_day

-- Vytvoreni tabulky
create table weather_day
(id serial primary key,
 place varchar(80) not null,
 meteo_date date not null,
 meteo_vals xml not null,
 unique(place,meteo_date));

-- Vložení dat za Vevěří a Prahu 21.3.2016
insert into weather_day
(place,meteo_date,meteo_vals)
values
('Veveri',
to_date('21.3.2016','DD.MM.YYYY'),
'<meteo>
<hourly><time>9</time><temp>6</temp><feels>3</feels><humidity>61</humidity></hourly>
<hourly><time>10</time><temp>7</temp><feels>5</feels><humidity>55</humidity></hourly>
<hourly><time>11</time><temp>8</temp><feels>5</feels><humidity>52</humidity></hourly>
<hourly><time>12</time><temp>8</temp><feels>5</feels><humidity>53</humidity></hourly>
<hourly><time>13</time><temp>8</temp><feels>6</feels><humidity>52</humidity></hourly>
<hourly><time>14</time><temp>9</temp><feels>6</feels><humidity>52</humidity></hourly>
<hourly><time>15</time><temp>9</temp><feels>6</feels><humidity>52</humidity></hourly>
<hourly><time>16</time><temp>8</temp><feels>6</feels><humidity>55</humidity></hourly>
</meteo>'::XML),
('Prague',
to_date('21.3.2016','DD.MM.YYYY'),
'<meteo>
<hourly><time>9</time><temp>5</temp><feels>2</feels><humidity>73</humidity></hourly>
<hourly><time>10</time><temp>6</temp><feels>3</feels><humidity>69</humidity></hourly>
<hourly><time>11</time><temp>7</temp><feels>4</feels><humidity>65</humidity></hourly>
<hourly><time>12</time><temp>7</temp><feels>5</feels><humidity>61</humidity></hourly>
<hourly><time>13</time><temp>8</temp><feels>6</feels><humidity>59</humidity></hourly>
<hourly><time>14</time><temp>8</temp><feels>6</feels><humidity>59</humidity></hourly>
<hourly><time>15</time><temp>8</temp><feels>6</feels><humidity>58</humidity></hourly>
<hourly><time>16</time><temp>8</temp><feels>6</feels><humidity>59</humidity></hourly>
</meteo>'::XML);

 
-- Výběr dat z celé tabulky

select
place, 
meteo_date +(array_to_string(xpath('/hourly/time/text()'::text, radek),',')::text||' hours')::interval as hour, 
array_to_string(xpath('/hourly/temp/text()'::text, radek),',')::integer as zkr,
array_to_string(xpath('/hourly/feels/text()'::text, radek),',')::integer as koeficient,  
array_to_string(xpath('/hourly/humidity/text()'::text, radek),',')::integer as koeficient  
from 
(select unnest(xpath('/meteo/hourly'::text,meteo_vals)) as radek,meteo_date,place
 from weather_day ) a;


-- Zrušení tabulky
 drop table weather_day