﻿/*
*
*  Kompozitní datové typy
*
*/

-- ===============================
-- Vyvoření kompozitního datového typu
-- ===============================

create type t_user as
(first_name varchar(80),
 last_name varchar(80),
 personal_no char(8));

 -- Použití v tabulce
  create table user_employee
 (id serial primary key,
  employee t_user,
  date_input date,
  date_outup date);

-- Pro zabespečení referenční integrity jedinečné osobní číslo
CREATE UNIQUE INDEX idx ON user_employee (((employee).personal_no));

-- Příklad vložení dat odkazem na kompositní sloupec "employee"
INSERT INTO public.user_employee(employee, date_input, date_outup)
  VALUES ( ('Michal','aaa',15) , now(), null);

-- Příklad vložení dat odkazem na atributy kompositního sloupce
INSERT INTO public.user_employee(
            employee.first_name,employee.last_name,employee.personal_no,date_input, date_outup)
    VALUES ( 'Petr','bbb',23, now(), null);    

-- Mněl by zafungovat  jedinečný index (Pokus o vložení stejného osobního čísla)
INSERT INTO public.user_employee(employee, date_input, date_outup)
  VALUES ( ('Bruno','sss','15') , now(), null);

/*
ERROR:  duplicate key value violates unique constraint "idx"
DETAIL:  Key (((employee).personal_no))=(15      ) already exists.
********** Chyba **********

ERROR: duplicate key value violates unique constraint "idx"
Stav SQL: 23505
Podrobnosti:Key (((employee).personal_no))=(15      ) already exists.
*/

-- Výběr dat
select * from user_employee;
/*
 id |        employee         | date_input | date_outup
----+-------------------------+------------+------------
  1 | (Michal,aaa,"15      ") | 2016-03-31 |
  2 | (Petr,bbb,"23      ")   | 2016-03-31 |
(2 řádky)
*/

-- Výběr dat specifikací výčtem atributů 
select (employee).first_name,(employee).last_name,date_input from user_employee;

/*
 first_name | last_name | date_input
------------+-----------+------------
 Michal     | aaa       | 2016-03-31
 Petr       | bbb       | 2016-03-31
(2 řádky)
*/

/*
drop type t_user cascade;
drop table user_employee;
*/
