﻿/*
*
*   Cizí klíč
*
*/

drop table if exists kniha_zarazeni cascade;
drop table if exists kategorie cascade;
drop table if exists kniha cascade;
 
-- Vytvoření tabulky kategorie 
create table kategorie
(id serial primary key,
  nazev_kategorie varchar(80));

-- Naplnění daty
insert into kategorie
(nazev_kategorie)
values
('americká literatura'),
('biografie'),
('česká literatura pro děti'),
('dobrodružná literatura'),
('společenská literatúra'),
('Edmondo'),
('Verne'),
('francouzská literatura'),
('fancouzští prozaici')
returning *;



-- Vytvoření tabulky knihy
create table kniha
(id serial primary key,
 nazev_knihy varchar(80) not null unique,
 rok_vydani smallint not null,
 anotace text,
 status varchar(30) not null);

insert into kniha
(nazev_knihy,rok_vydani,status)
values
('Cesta do středu Země',2015,'aktivní'),
('Cesta kolem světa za 80 dní',2015,'aktivní'), 
('Tajuplný ostrov',1995,'aktivní'),
('Srdce',2006,'aktivní'),
('Dub a květinka',1910,'aktivní'),
('Albert',1900,'ztraceno');

-- Vytvoření tabulky kniha_zarazení
-- s referenční integritou
create table kniha_zarazeni
(kategorie_id integer not null references kategorie(id),
 kniha_id integer not null references kniha(id),
 primary key (kategorie_id,kniha_id)
);

-- Vložení dat 
insert into kniha_zarazeni
(kniha_id,kategorie_id)
values
(1,4),(1,8),(1,9),(1,7),
(2,4),(2,8),(2,9),(2,7),
(3,4),(3,8),(3,9),(3,7),
(4,5),(4,8),(4,9),(4,6),
(5,5),(5,8),(5,9),(5,6),
(6,5),(6,8),(6,9),(6,6);


-- Pokus o vložení špatných dat neodpovidají cizím klíčům

insert into kniha_zarazeni
(kniha_id,kategorie_id)
values
(12,23);

/*
ERROR:  insert or update on table "kniha_zarazeni" violates foreign key constraint "kniha_zarazeni_kategorie_id_fkey"
DETAIL:  Key (kategorie_id)=(23) is not present in table "kategorie".
********** Chyba **********

ERROR: insert or update on table "kniha_zarazeni" violates foreign key constraint "kniha_zarazeni_kategorie_id_fkey"
Stav SQL: 23503
*/

-- Vytvoření uživatelkého pohledu view
create view v_kniha_zarazeni
as
select nazev_knihy, nazev_kategorie
from kniha_zarazeni kz
join kniha kn on kz.kniha_id = kn.id
join kategorie ka on kz.kategorie_id = ka.id;



-- Vyber dat
select * from v_kniha_zarazeni;

-- Uživatelský dotaz knihy Verne
select nazev_knihy 
from v_kniha_zarazeni
where nazev_kategorie = 'Verne';

/*
"Cesta do středu Země"
"Cesta kolem světa za 80 dní"
"Tajuplný ostrov"
*/

-- Uživatelský dotaz knihy francouzské
select nazev_knihy 
from v_kniha_zarazeni
where nazev_kategorie = 'francouzská literatura';

/*
"Cesta do středu Země"
"Cesta kolem světa za 80 dní"
"Tajuplný ostrov"
*/

-- Oprava v tabulce katagorie
update kategorie
set nazev_kategorie = 'dobrodružná literatúra'
where nazev_kategorie = 'francouzská literatura';

-- Vypis chybove hlasky pri pokusu zrusit cizího klíču
delete from kategorie where nazev_kategorie = 'Verne';

/*
ERROR:  update or delete on table "kategorie" violates foreign key constraint "kniha_zarazeni_kategorie_id_fkey" on table "kniha_zarazeni"
DETAIL:  Key (id)=(7) is still referenced from table "kniha_zarazeni".
********** Chyba **********

ERROR: update or delete on table "kategorie" violates foreign key constraint "kniha_zarazeni_kategorie_id_fkey" on table "kniha_zarazeni"
Stav SQL: 23503
Podrobnosti:Key (id)=(7) is still referenced from table "kniha_zarazeni".
*/





