﻿/*
*
*  CHECK kontrola vstupních dat
*
*/


-- ================================
-- Kontrola - vstupních hodnot
-- ================================

-- Datum vstupu musí být menší nebo roven jako je dnešní datum

drop table if exists ctenar;
create table ctenar
(id serial primary key, 
 jmeno varchar(60) not null,
 prijmeni varchar(80) not null,
 datum_vstupu date not null check (datum_vstupu <= current_date));

-- Vlozeni spravneho zaznamu
 insert into ctenar
 (jmeno,prijmeni,datum_vstupu)
 values
 ('Ivan','Hrozný','2016-03-02')
 returning *;
 
/*
1;"Ivan";"Hrozný";"2016-03-02"
*/

-- Vložení záznamu s nesprávným datumem
  insert into ctenar
 (jmeno,prijmeni,datum_vstupu)
 values
 ('Petr','Veliký','2016-08-02');

 /*
 ERROR:  new row for relation "ctenar" violates check constraint "ctenar_datum_vstupu_check"
DETAIL:  Failing row contains (2, Petr, Veliký, 2016-08-02).
********** Chyba **********
*/

 -- ========================================
 -- Kontrola - v rámci řádku
 -- ========================================

 -- Přidáne k tabulce sloupec datum_vystupu
 -- Pravidlo datum_vystupu >= datum_vstupu

 alter table ctenar 
 add column datum_vystupu date 
 check (datum_vystupu >= datum_vstupu);


-- Aktualizace ze špatným datumem
update ctenar
   set datum_vystupu = '2009-05-15'
where id = 1;

/*
ERROR:  new row for relation "ctenar" violates check constraint "ctenar_check"
DETAIL:  Failing row contains (1, Ivan, Hrozný, 2016-03-02, 2009-05-15).
********** Chyba **********
*/


-- drop table if exists ctenar;

-- ================================================
-- Kontrola - v rámci řádku komplexně celého řádku
-- ================================================


-- Vytvoří se tabulka, která bude identifikovat zdroj dat jeden ze tří

drop table if exists pozadavek;
create table pozadavek
(id serial primary key,
 zdroj1_id integer,
 zdroj2_id integer,
 zdroj3_id integer,
 txt text,
 check (
 (zdroj1_id is not null and zdroj2_id is null and zdroj3_id is null) or
 (zdroj1_id is null and zdroj2_id is not null and zdroj3_id is null) or
 (zdroj1_id is null and zdroj2_id is null and zdroj3_id is not null) ));
/*
 Jsou zde uvedeny všechny platné kombinace.
*/


-- Neprojde kontrolou 
insert into pozadavek
(zdroj1_id,zdroj2_id,zdroj3_id,txt)
values
(132,3213,0,'špatný příklad');
/*
ERROR:  new row for relation "pozadavek" violates check constraint "pozadavek_check"
DETAIL:  Failing row contains (1, 132, 3213, 0, špatný příklad).
********** Chyba **********
*/

-- Projde kontrolou
insert into pozadavek
(zdroj1_id,zdroj2_id,zdroj3_id,txt)
values
(132,null,null,'příklad OK');


-- Check nad řádkem je možné ladit i tak , že se sestavuje select nad zadanými daty
-- Zde alternativa k předchozímu příkladu
 select id,
 ARRAY[zdroj1_id is null, zdroj2_id is null, zdroj3_id is null]
 in (ARRAY[false,true,true], ARRAY[true,false,true], ARRAY[true,true,false])
 from pozadavek;


  

 
  