﻿/*
*
*  Unique
*
*/

drop table if exists color;

create table color
(id serial primary key,
 color varchar(60) not null unique,
 rgb_1 smallint not null,
 rgb_2 smallint not null,
 rgb_3 smallint not null,
 unique(rgb_1,rgb_2,rgb_3));

insert into color
(color,rgb_1,rgb_2,rgb_3)
values
('Red',255,0,0),
('Orange',255,255,0),
('Yellow',0,255,0),
('Cyan',0,255,255),
('Blue',0,0,255)
returning *;

/*
1;"Red";255;0;0
2;"Orange";255;255;0
3;"Yellow";0;255;0
4;"Cyan";0;255;255
5;"Blue";0;0;255
*/


/*

Upozornění
==========
Nasledující příklad pouze demonstruje různé techniky cizích klíčů.
Vpraxi umožňuje insert a update anomalitu. Jiný color a jiný RGB

*/
-- na unique constrainy je možné vázat taky cizí klíče
-- P59klad jednoduché reference i složené

create table color_pallete
(id serial primary key,
 color varchar(60) not null unique references color(color),
 rgb_1 smallint not null,
 rgb_2 smallint not null,
 rgb_3 smallint not null,
 foreign key (rgb_1,rgb_2,rgb_3) references color(rgb_1,rgb_2,rgb_3) 
 );


-- Správně vložená hodnota

 insert into color_pallete
(color,rgb_1,rgb_2,rgb_3)
values
('Red',255,0,0);


-- Špatně vložená hodnota

 insert into color_pallete
(color,rgb_1,rgb_2,rgb_3)
values
('Yellow',255,1,0);
 
/*
ERROR:  insert or update on table "color_pallete" violates foreign key constraint "color_pallete_rgb_1_fkey"
DETAIL:  Key (rgb_1, rgb_2, rgb_3)=(255, 1, 0) is not present in table "color".
********** Chyba **********

ERROR: insert or update on table "color_pallete" violates foreign key constraint "color_pallete_rgb_1_fkey"
Stav SQL: 23503
Podrobnosti:Key (rgb_1, rgb_2, rgb_3)=(255, 1, 0) is not present in table "color".
*/

 -- =====================================
 -- Jak zabespečit null jedinečnost
 -- =====================================
-- Null jedinečnost neexistuje

 -- Máme příklad
 drop table if exists test1;
 create table test1(
  id serial primary key,
  txt text not null,
  kl1 integer,
  kl2 integer,
  kl3 integer,
  unique(kl1,kl2,kl3)
 );

 -- Vložíme záznami
 insert into test1
 (txt,kl1,kl2,kl3)
 values
 (1,1,2,3),
 (2,null,null,null),
 (3,null,null,null);

